Package: paramix
Title: Aggregate and Disaggregate Continuous Parameters for
        Compartmental Models
Version: 0.0.2
Authors@R: 
    c(person("Carl", "Pearson", , "carl.ab.pearson@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0701-7860")),
      person("Simon", "Proctor", , "simon.procter@lshtm.ac.uk", role = c("aut"),
           comment = c(ORCID = "0000-0002-0380-1503")),
      person("Lucy", "Goodfellow", , "lucy.goodfellow@lshtm.ac.uk", role = c("aut"),
           comment = c(ORCID = "0009-0004-0434-5863")))
Description: A convenient framework for aggregating and disaggregating continuously
    varying parameters (for example, case fatality ratio, with age) for proper
    parametrization of lower-resolution compartmental models (for example, with
    broad age categories) and subsequent upscaling of model outputs to high resolution
    (for example, as needed when calculating age-sensitive measures like years-life-lost).
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
Suggests: lintr, knitr, rmarkdown, ggplot2, roxygen2, wpp2019, testthat
        (>= 3.0.0), patchwork
VignetteBuilder: knitr
Imports: data.table
Config/testthat/edition: 3
URL: https://cmmid.github.io/paramix/
NeedsCompilation: no
Packaged: 2025-06-10 14:21:42 UTC; carl
Author: Carl Pearson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0701-7860>),
  Simon Proctor [aut] (ORCID: <https://orcid.org/0000-0002-0380-1503>),
  Lucy Goodfellow [aut] (ORCID: <https://orcid.org/0009-0004-0434-5863>)
Maintainer: Carl Pearson <carl.ab.pearson@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-10 14:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 21:03:15 UTC; unix
