# Generated by roxygen2: do not edit by hand

export(alembic)
export(blend)
export(distill)
export(distill_summary)
export(interpolate_opts)
export(parameter_summary)
import(data.table)
importFrom(data.table,":=")
importFrom(data.table,.BY)
importFrom(data.table,.EACHI)
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.NGRP)
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(data.table,melt.data.table)
importFrom(stats,integrate)
importFrom(stats,weighted.mean)
importFrom(utils,head)
importFrom(utils,tail)
